from cassandra.model.linear import linear
from cassandra.model.logLinear import logLinear
from cassandra.model.ridge import ridge
from cassandra.model.logLog import logLog


def choose_model(df, X_columns, y_column, model_regression='linear', X_trasformations_columns=[],
                model_regression_log_log='linear', ridge_number=0, metric=['rsq', 'nrmse', 'mape'],
                return_metric=False, size=0.2, positive=False, random_state=42, force_coeffs=False, coeffs=[], intercept=0):
    metrics_values = {}

    if 'linear' in model_regression:
        if return_metric:
            result, model, metrics_values = linear(df, X_columns, y_column, model_regression, metric=metric,
                                                   return_metric=return_metric, size=size, positive=positive,
                                                   random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)
        else:
            result, model = linear(df, X_columns, y_column, model_regression, metric=metric, return_metric=return_metric,
                                   size=size, positive=positive, random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)

    elif 'logLinear' in model_regression:
        if return_metric:
            result, model, metrics_values = logLinear(df, X_columns, y_column, model_regression, metric=metric,
                                                      return_metric=return_metric, size=size, positive=positive,
                                                      random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)
        else:
            result, model = logLinear(df, X_columns, y_column, model_regression, metric=metric, return_metric=return_metric,
                                      size=size, positive=positive, random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)

    elif 'logLog' in model_regression:
        if return_metric:
            result, model, metrics_values = logLog(df, X_trasformations_columns, X_columns, y_column, model_regression,
                                                   model_regression=model_regression_log_log, metric=metric,
                                                   return_metric=return_metric, size=size, positive=positive,
                                                   random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)
        else:
            result, model = logLog(df, X_trasformations_columns, X_columns, y_column, model_regression,
                                   model_regression=model_regression_log_log, metric=metric,
                                   return_metric=return_metric, size=size, positive=positive, random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)

    elif 'ridge' in model_regression:
        if return_metric:
            result, model, metrics_values = ridge(df, X_columns, y_column, model_regression, ridge_number=ridge_number,
                                                  metric=metric,
                                                  return_metric=return_metric, size=size, positive=positive,
                                                  random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)
        else:
            result, model = ridge(df, X_columns, y_column, model_regression, ridge_number=ridge_number, metric=metric,
                                  return_metric=return_metric, size=size, positive=positive, random_state=random_state, force_coeffs=force_coeffs, coeffs=coeffs, intercept=intercept)

    if metrics_values:
        return result, model, metrics_values
    else:
        return result, model



#!/usr/bin/env python
# coding: utf-8

# In[1]:


# Imports
import pandas as pd
import numpy as np
import nevergrad as ng

from sklearn.linear_model import Ridge
from sklearn.linear_model import LinearRegression
from sklearn.model_selection import train_test_split
from sklearn.neural_network import MLPRegressor

from src.cassandra.model.linear import *
from src.cassandra.model.logLinear import *
from src.cassandra.model.logLog import *
from src.cassandra.model.mlpRegressor import *
from src.cassandra.model.nevergrad.nevergrad import *
from src.cassandra.model.ridge import *
from src.cassandra.model.modelEvaluation.plot import *
from src.cassandra.model.nestedModel import *
from src.cassandra.data.dataProcessing.prophet import *
from src.cassandra.data.trasformations.trasformations import *
from src.cassandra.model.modelEvaluation.evaluation import *
from src.cassandra.model.nevergrad import *
from src.cassandra.data.dataAnalysis.plot import *
from src.cassandra.data.dataProcessing.clean_and_format import *
from src.cassandra.data.featureSelection.ols import *
from src.cassandra.budgetAllocator.budget_allocator import *
from src.cassandra.budgetAllocator.utils import *
from src.cassandra.utils.save import *

# In[2]:


# pip install cassandra-mmm --upgrade


# In[25]:


subset_columns = ['date', 'conversions', 'fb_pr_spend', 'fb_re_spend',
                  'g_discovery_spend', 'g_display_spend', 'g_performance_max_spend', 'g_search_spend', 'g_video_spend',
                  'tiktok_learn_spend', 'influencer_spend', 'email_campaigns_sessions',
                  's_organic', 's_referral', 's_influencer', 's_affiliate', 's_direct', 's_partner', 's_mediapartner',
                  'adopta_albero_impressions', 'regalar_arbol_impressions', 'regala_arbol_impressions',
                  'plantar_arbol_impressions', 'treedom_impressions',
                  # 'avg_p_cacao', 'avg_p_baobab', 'avg_p_asc2020', 'avg_p_pt2020', 'avg_p_ft_ag',
                  'tickets_count']
csv_data = create_dataset('D:/workspace_cassandra/mmm/treedom/ES/treedom-es - modifiche.csv', subset_columns=subset_columns)

# csv_data = create_data_columns(csv_data, 'date', national_holidays_abbreviation = 'ES')

# csv_data.drop(csv_data[csv_data['date'] < '2021-02-28'].index, inplace=True)
# csv_data.drop(csv_data[csv_data['conversions'] < 1].index, inplace=True)
# csv_data.drop(csv_data[csv_data['conversions'] > 55].index, inplace=True)

# pd.set_option("display.max_rows", None)
# pd.set_option("display.max_columns", None)
pd.set_option("display.max_rows", 20)
pd.set_option("display.max_columns", 20)
csv_data.fillna(method='bfill', inplace=True)
csv_data.fillna(method='ffill', inplace=True)
csv_data.fillna(0, inplace=True)
csv_data

# # Data Preparation

# In[26]:


keyword_cols_aggregate = ['adopta_albero_impressions', 'regalar_arbol_impressions', 'regala_arbol_impressions',
                          'plantar_arbol_impressions', 'treedom_impressions']
csv_data['keywords_impressions'] = csv_data[keyword_cols_aggregate].sum(axis=1)

csv_data.drop(keyword_cols_aggregate, axis=1, inplace=True)

s_mediapartner_aggregate = ['s_mediapartner', 's_partner']
csv_data['s_partners'] = csv_data[s_mediapartner_aggregate].sum(axis=1)

csv_data.drop(s_mediapartner_aggregate, axis=1, inplace=True)

s_influencer_aggregate = ['s_influencer', 's_affiliate']
csv_data['s_influencers'] = csv_data[s_influencer_aggregate].sum(axis=1)

csv_data.drop(s_influencer_aggregate, axis=1, inplace=True)

s_other_aggregate = ['email_campaigns_sessions', 's_partners']
csv_data['s_others'] = csv_data[s_other_aggregate].sum(axis=1)

csv_data.drop(s_other_aggregate, axis=1, inplace=True)

# products_vars = [col for col in df.columns if 'avg' in col]

# for p in products_vars:
# for index, row in csv_data.iterrows():
# if row[p] != 0:
# csv_data.at[index, p] = 1
# else:
# csv_data.at[index, p] = 0


# # Prophet

# In[27]:


df = prophet(csv_data, 'date', 'conversions', window_start='2021-02-28', national_holidays_abbreviation='ES',
             future_dataframe_periods=365, plot_prophet=True, daily_seasonality=False)

# In[28]:


df

# # Spends

# In[7]:


medias_google = [col for col in df.columns if 'spend' in col if 'g_' in col]
medias_fb = [col for col in df.columns if 'spend' in col if 'fb_' in col]
medias_others = [col for col in df.columns if 'spend' in col if 'fb_' not in col if 'g_' not in col]

media = medias_google + medias_fb + medias_others

spends_df = spend_to_dataset(df, media)
spends_df

# In[8]:


spends, response_get_budget = getBudget(df, 'date', 'conversions', media, '2022-02-28')
print(spends, np.sum(spends), response_get_budget)

# # Correlation

# In[29]:


pd.set_option("display.max_rows", None)
pd.set_option("display.max_columns", None)
df.corr()

# In[51]:


organic = ['s_organic', 's_referral', 's_influencers', 's_direct', 's_others', 'keywords_impressions']
context = ['tickets_count']

# product_vars = [col for col in df.columns if 'avg' in col]

prophet_vars = ['trend', 'yearly', 'monthly', 'weekly', 'holidays']
# prophet_vars = ['day_week', 'month', 'festivo']

all_features = media + organic + context + prophet_vars  # + product_vars

# In[52]:


X = df[all_features]
y = df['conversions']

result_ols, model_ols = ols(X, y, constant=True)

# # Nested Model

# In[32]:


# Sub-model describes Keywords Impressions using Google and Facebook's Media Spents
# Mail model uses all other variables and the sub-model to describe conversions
# medias_main = medias_google + medias_fb

# result, model, metrics_values, sub_result, sub_model, sub_metric_values = mainNestedModel(df, medias_main, 'keywords_impressions', all_variables, 'conversions', sub_model_regression = 'linear', sub_metric = ['rsq', 'nrmse', 'mape'], return_sub_metric = True, model_regression = 'ridge', metric = ['rsq', 'nrmse', 'mape'], return_metric = True)


# # Ridge Regression

# In[53]:


result_ridge, model_ridge = ridge(df, X.columns, 'conversions', 'Ridge', ridge_number=21,
                                  metric=['rsq', 'nrmse', 'mape'])

# In[55]:


#show_prediction_vs_actual_graph_with_error(result_ridge, result_ridge['conversions'], result_ridge['prediction'],
                                           #'date', 'revenue', name_model='')

# In[56]:


#show_coefficients(all_features, model_ridge, 'Ridge')

# # Linear Regression

# In[36]:


#result_linear, model_linear = linear(df, X.columns, 'conversions', 'Linear', metric=['rsq', 'nrmse', 'mape'])

# In[37]:


#show_prediction_vs_actual_graph_with_error(result_linear, result_linear['conversions'], result_linear['prediction'],
                                           #'date', 'conversions', name_model='')

# In[38]:


#show_coefficients(all_features, model_linear, 'Linear')

# # LogLog Linear Regression

# In[39]:


trasformations_log_columns = media + organic
not_log_columns = prophet_vars + context

# In[40]:


#result_logLog_linear, model_logLog_linear = logLog(df, trasformations_log_columns, not_log_columns, 'conversions',
                                                   #'logLog', model_regression='linear', metric=['rsq', 'nrmse', 'mape'])

# In[41]:


# show_prediction_vs_actual_graph(result_logLog, 'date', 'conversions', 'prediction')


# In[42]:


#show_prediction_vs_actual_graph_with_error(result_logLog_linear, result_logLog_linear['conversions'],
                                           #result_logLog_linear['prediction'], 'date', 'conversions', name_model='')

# In[43]:


#show_coefficients(all_features, model_logLog_linear, 'Log Log')

# # LogLog Ridge Regression

# In[44]:


#result_logLog_ridge, model_logLog_ridge = logLog(df, trasformations_log_columns, not_log_columns, 'conversions',
                                                 #'logLog', model_regression='ridge', metric=['rsq', 'nrmse', 'mape'])

# In[45]:


#show_prediction_vs_actual_graph_with_error(result_logLog_ridge, result_logLog_ridge['conversions'],
                                           #result_logLog_ridge['prediction'], 'date', 'conversions', name_model='')

# In[46]:


#show_coefficients(all_features, model_logLog_ridge, 'Log Log Ridge')

# # Modello Ridge

# In[47]:

hyperparameters_dict = instrum_variables_nevergrad(media, organic)

metric, result, model, metric_array = nevergrad_model(df, hyperparameters_dict, media, organic, 'conversions')

# def build_model(b1, b2, b3, b4, b5, b6, b7, b8, b11, b13, b14, b15, b16, b17, b18, b19,
#                 b20, b21, b22, b23, b24, b0,  # b26, b27, b28, b29, b30, b31, b32, b33,
#                 g_discovery_spend_scale, g_discovery_spend_shape, g_discovery_spend_beta,
#                 g_display_spend_scale, g_display_spend_shape, g_display_spend_beta,
#                 g_performance_max_spend_scale, g_performance_max_spend_shape, g_performance_max_spend_beta,
#                 g_search_spend_scale, g_search_spend_shape, g_search_spend_beta,
#                 g_video_spend_scale, g_video_spend_shape, g_video_spend_beta,
#                 fb_re_spend_scale, fb_re_spend_shape, fb_re_spend_beta,
#                 fb_pr_spend_scale, fb_pr_spend_shape, fb_pr_spend_beta,
#                 tiktok_learn_spend_scale, tiktok_learn_spend_shape, tiktok_learn_spend_beta,
#                 # microsoft_spend_scale, microsoft_spend_shape, microsoft_spend_beta,
#                 # linkedin_spend_scale, linkedin_spend_shape, linkedin_spend_beta,
#                 influencer_spend_scale, influencer_spend_shape, influencer_spend_beta):
#     # Google
#     x1 = df['g_discovery_spend']
#     x2 = df['g_display_spend']
#     x3 = df['g_performance_max_spend']
#     x4 = df['g_search_spend']
#     x5 = df['g_video_spend']
#
#     x1_adstock = adstock_weibull(x1, g_discovery_spend_scale, g_discovery_spend_shape).replace([np.inf, -np.inf], 0)
#     x2_adstock = adstock_weibull(x2, g_display_spend_scale, g_display_spend_shape).replace([np.inf, -np.inf], 0)
#     x3_adstock = adstock_weibull(x3, g_performance_max_spend_scale, g_performance_max_spend_shape).replace(
#         [np.inf, -np.inf], 0)
#     x4_adstock = adstock_weibull(x4, g_search_spend_scale, g_search_spend_shape).replace([np.inf, -np.inf], 0)
#     x5_adstock = adstock_weibull(x5, g_video_spend_scale, g_video_spend_shape).replace([np.inf, -np.inf], 0)
#
#     x1 = saturation(x1_adstock, g_discovery_spend_beta)
#     x2 = saturation(x2_adstock, g_display_spend_beta)
#     x3 = saturation(x3_adstock, g_performance_max_spend_beta)
#     x4 = saturation(x4_adstock, g_search_spend_beta)
#     x5 = saturation(x5_adstock, g_video_spend_beta)
#
#     # Facebook
#     x6 = df['fb_re_spend']
#     x7 = df['fb_pr_spend']
#
#     x6_adstock = adstock_weibull(x6, fb_re_spend_scale, fb_re_spend_shape).replace([np.inf, -np.inf], 0)
#     x7_adstock = adstock_weibull(x7, fb_pr_spend_scale, fb_pr_spend_shape).replace([np.inf, -np.inf], 0)
#
#     x6 = saturation(x6_adstock, fb_re_spend_beta)
#     x7 = saturation(x7_adstock, fb_pr_spend_beta)
#
#     # Other Medias
#     x8 = df['tiktok_learn_spend']
#     # x9 = df['microsoft_spend']
#     # x10 = df['linkedin_spend']
#     x11 = df['influencer_spend']
#
#     x8_adstock = adstock_weibull(x8, tiktok_learn_spend_scale, tiktok_learn_spend_shape).replace([np.inf, -np.inf], 0)
#     # x9_adstock = adstock_weibull(x9, microsoft_spend_scale, microsoft_spend_shape).replace([np.inf, -np.inf], 0)
#     # x10_adstock = adstock_weibull(x10, linkedin_spend_scale, linkedin_spend_shape).replace([np.inf, -np.inf], 0)
#     x11_adstock = adstock_weibull(x11, influencer_spend_scale, influencer_spend_shape).replace([np.inf, -np.inf], 0)
#
#     x8 = saturation(x8_adstock, tiktok_learn_spend_beta)
#     # x9 = saturation(x9_adstock, microsoft_spend_beta)
#     # x10 = saturation(x10_adstock, linkedin_spend_beta)
#     x11 = saturation(x11_adstock, influencer_spend_beta)
#
#     # Google
#     df_transf['g_discovery_spend'] = x1
#     df_transf['g_display_spend'] = x2
#     df_transf['g_performance_max_spend'] = x3
#     df_transf['g_search_spend'] = x4
#     df_transf['g_video_spend'] = x5
#
#     # Facebook
#     df_transf['fb_re_spend'] = x6
#     df_transf['fb_pr_spend'] = x7
#
#     # Other Medias
#     df_transf['tiktok_learn_spend'] = x8
#     # df_transf['microsoft_spend'] = x9
#     # df_transf['linkedin_spend'] = x10
#     df_transf['influencer_spend'] = x11
#
#     X = df_transf[all_features]
#     y = df_transf['conversions']
#
#     X_train, X_test, y_train, y_test = train_test_split(X, y)
#
#     model = Ridge(22)
#
#     model.fit(X_train, y_train)
#
#     coefs = [b1] + [b2] + [b3] + [b4] + [b5] + [b6] + [b7] + [b8] + [b11] + [b13] + [b14] + [b15] + [b16] + [b17] + [
#         b18] + [b19] + [b20] + [b21] + [b22] + [b23] + [b24]
#     model.coef_ = np.array(coefs)
#     model.intercept_ = b0
#
#     # Setting the Result Dataset
#     result = df_transf.copy()
#
#     result['prediction'] = model.predict(X)
#
#     nrmse_val = show_nrmse(result['conversions'], result['prediction'])
#     mape_val = show_mape(result['conversions'], result['prediction'])
#     rsquared_val = show_rsquared(result['conversions'], result['prediction'])
#
#     print(rsquared_val, nrmse_val, mape_val)
#
#     mape_array.append(mape_val)
#
#     return mape_val
#
#
# # In[48]:
#
#
# instrum = ng.p.Instrumentation(
#     # alpha = ng.p.Scalar(lower=0),
#
#     b1=ng.p.Scalar(lower=0),
#     b2=ng.p.Scalar(lower=0),
#     b3=ng.p.Scalar(lower=0),
#     b4=ng.p.Scalar(lower=0),
#     b5=ng.p.Scalar(lower=0),
#     b6=ng.p.Scalar(lower=0),
#     b7=ng.p.Scalar(lower=0),
#     b8=ng.p.Scalar(lower=0),
#     # b9 = ng.p.Scalar(lower=0),
#     # b10 = ng.p.Scalar(lower=0),
#     b11=ng.p.Scalar(lower=0),
#
#     # b12 = ng.p.Scalar(),
#     b13=ng.p.Scalar(lower=0),
#     b14=ng.p.Scalar(lower=0),
#     b15=ng.p.Scalar(lower=0),
#     b16=ng.p.Scalar(lower=0),
#     b17=ng.p.Scalar(lower=0),
#     b18=ng.p.Scalar(lower=0),
#     b19=ng.p.Scalar(upper=0),
#     b20=ng.p.Scalar(),
#     b21=ng.p.Scalar(),
#     b22=ng.p.Scalar(),
#     b23=ng.p.Scalar(),
#     b24=ng.p.Scalar(),
#     # b25 = ng.p.Scalar(),
#     # b26 = ng.p.Scalar(),
#     # b27 = ng.p.Scalar(),
#     # b28 = ng.p.Scalar(),
#     # b29 = ng.p.Scalar(),
#     # b30 = ng.p.Scalar(),
#     # b31 = ng.p.Scalar(),
#     # b32 = ng.p.Scalar(),
#     # b33 = ng.p.Scalar(),
#
#     b0=ng.p.Scalar(),
#
#     g_discovery_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     g_discovery_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     g_discovery_spend_beta=ng.p.Scalar(lower=0, upper=1),
#     g_display_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     g_display_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     g_display_spend_beta=ng.p.Scalar(lower=0, upper=1),
#     g_performance_max_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     g_performance_max_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     g_performance_max_spend_beta=ng.p.Scalar(lower=0, upper=1),
#     g_search_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     g_search_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     g_search_spend_beta=ng.p.Scalar(lower=0, upper=1),
#     g_video_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     g_video_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     g_video_spend_beta=ng.p.Scalar(lower=0, upper=1),
#
#     fb_re_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     fb_re_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     fb_re_spend_beta=ng.p.Scalar(lower=0, upper=1),
#     fb_pr_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     fb_pr_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     fb_pr_spend_beta=ng.p.Scalar(lower=0, upper=1),
#
#     tiktok_learn_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     tiktok_learn_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     tiktok_learn_spend_beta=ng.p.Scalar(lower=0, upper=1),
#
#     # microsoft_spend_scale = ng.p.Scalar(lower=0, upper=0.1),
#     # microsoft_spend_shape = ng.p.Scalar(lower=0, upper=2),
#     # microsoft_spend_beta = ng.p.Scalar(lower=0, upper=1),
#
#     # linkedin_spend_scale = ng.p.Scalar(lower=0, upper=0.1),
#     # linkedin_spend_shape = ng.p.Scalar(lower=0, upper=2),
#     # linkedin_spend_beta = ng.p.Scalar(lower=0, upper=1),
#
#     influencer_spend_scale=ng.p.Scalar(lower=0, upper=0.1),
#     influencer_spend_shape=ng.p.Scalar(lower=0, upper=2),
#     influencer_spend_beta=ng.p.Scalar(lower=0, upper=1)
# )
#
# optimizer = ng.optimizers.TwoPointsDE(parametrization=instrum, budget=1000)
# recommendation = optimizer.minimize(build_model)
#
# # NGOpt
# # TwoPointsDE
# recommendation.value
#
#
# # In[49]:
#
#
# def build_model(b1, b2, b3, b4, b5, b6, b7, b8, b11, b13, b14, b15, b16, b17, b18, b19,
#                 b20, b21, b22, b23, b24, b0,  # b26, b27, b28, b29, b30, b31, b32, b33,
#                 g_discovery_spend_scale, g_discovery_spend_shape, g_discovery_spend_beta,
#                 g_display_spend_scale, g_display_spend_shape, g_display_spend_beta,
#                 g_performance_max_spend_scale, g_performance_max_spend_shape, g_performance_max_spend_beta,
#                 g_search_spend_scale, g_search_spend_shape, g_search_spend_beta,
#                 g_video_spend_scale, g_video_spend_shape, g_video_spend_beta,
#                 fb_re_spend_scale, fb_re_spend_shape, fb_re_spend_beta,
#                 fb_pr_spend_scale, fb_pr_spend_shape, fb_pr_spend_beta,
#                 tiktok_learn_spend_scale, tiktok_learn_spend_shape, tiktok_learn_spend_beta,
#                 # microsoft_spend_scale, microsoft_spend_shape, microsoft_spend_beta,
#                 # linkedin_spend_scale, linkedin_spend_shape, linkedin_spend_beta,
#                 influencer_spend_scale, influencer_spend_shape, influencer_spend_beta):
#     # Google
#     x1 = df['g_discovery_spend']
#     x2 = df['g_display_spend']
#     x3 = df['g_performance_max_spend']
#     x4 = df['g_search_spend']
#     x5 = df['g_video_spend']
#
#     x1_adstock = adstock_weibull(x1, g_discovery_spend_scale, g_discovery_spend_shape).replace([np.inf, -np.inf], 0)
#     x2_adstock = adstock_weibull(x2, g_display_spend_scale, g_display_spend_shape).replace([np.inf, -np.inf], 0)
#     x3_adstock = adstock_weibull(x3, g_performance_max_spend_scale, g_performance_max_spend_shape).replace(
#         [np.inf, -np.inf], 0)
#     x4_adstock = adstock_weibull(x4, g_search_spend_scale, g_search_spend_shape).replace([np.inf, -np.inf], 0)
#     x5_adstock = adstock_weibull(x5, g_video_spend_scale, g_video_spend_shape).replace([np.inf, -np.inf], 0)
#
#     x1 = saturation(x1_adstock, g_discovery_spend_beta)
#     x2 = saturation(x2_adstock, g_display_spend_beta)
#     x3 = saturation(x3_adstock, g_performance_max_spend_beta)
#     x4 = saturation(x4_adstock, g_search_spend_beta)
#     x5 = saturation(x5_adstock, g_video_spend_beta)
#
#     # Facebook
#     x6 = df['fb_re_spend']
#     x7 = df['fb_pr_spend']
#
#     x6_adstock = adstock_weibull(x6, fb_re_spend_scale, fb_re_spend_shape).replace([np.inf, -np.inf], 0)
#     x7_adstock = adstock_weibull(x7, fb_pr_spend_scale, fb_pr_spend_shape).replace([np.inf, -np.inf], 0)
#
#     x6 = saturation(x6_adstock, fb_re_spend_beta)
#     x7 = saturation(x7_adstock, fb_pr_spend_beta)
#
#     # Other Medias
#     x8 = df['tiktok_learn_spend']
#     # x9 = df['microsoft_spend']
#     # x10 = df['linkedin_spend']
#     x11 = df['influencer_spend']
#
#     x8_adstock = adstock_weibull(x8, tiktok_learn_spend_scale, tiktok_learn_spend_shape).replace([np.inf, -np.inf], 0)
#     # x9_adstock = adstock_weibull(x9, microsoft_spend_scale, microsoft_spend_shape).replace([np.inf, -np.inf], 0)
#     # x10_adstock = adstock_weibull(x10, linkedin_spend_scale, linkedin_spend_shape).replace([np.inf, -np.inf], 0)
#     x11_adstock = adstock_weibull(x11, influencer_spend_scale, influencer_spend_shape).replace([np.inf, -np.inf], 0)
#
#     x8 = saturation(x8_adstock, tiktok_learn_spend_beta)
#     # x9 = saturation(x9_adstock, microsoft_spend_beta)
#     # x10 = saturation(x10_adstock, linkedin_spend_beta)
#     x11 = saturation(x11_adstock, influencer_spend_beta)
#
#     # Google
#     df_transf['g_discovery_spend'] = x1
#     df_transf['g_display_spend'] = x2
#     df_transf['g_performance_max_spend'] = x3
#     df_transf['g_search_spend'] = x4
#     df_transf['g_video_spend'] = x5
#
#     # Facebook
#     df_transf['fb_re_spend'] = x6
#     df_transf['fb_pr_spend'] = x7
#
#     # Other Medias
#     df_transf['tiktok_learn_spend'] = x8
#     # df_transf['microsoft_spend'] = x9
#     # df_transf['linkedin_spend'] = x10
#     df_transf['influencer_spend'] = x11
#
#     X = df_transf[all_features]
#     y = df_transf['conversions']
#
#     X_train, X_test, y_train, y_test = train_test_split(X, y)
#
#     model = Ridge(22)
#
#     model.fit(X_train, y_train)
#
#     coefs = [b1] + [b2] + [b3] + [b4] + [b5] + [b6] + [b7] + [b8] + [b11] + [b13] + [b14] + [b15] + [b16] + [b17] + [
#         b18] + [b19] + [b20] + [b21] + [b22] + [b23] + [b24]
#     model.coef_ = np.array(coefs)
#     model.intercept_ = b0
#
#     # Setting the Result Dataset
#     result = df_transf.copy()
#
#     result['prediction'] = model.predict(X)
#
#     nrmse_val = show_nrmse(result['conversions'], result['prediction'])
#     mape_val = show_mape(result['conversions'], result['prediction'])
#     rsquared_val = show_rsquared(result['conversions'], result['prediction'])
#
#     print('RSQUARED: ', rsquared_val)
#     print('NRMSE: ', nrmse_val)
#     print('MAPE: ', mape_val)
#
#     return result, model
#
#
# result, model = build_model(**recommendation.value[1])

# In[24]:


show_mape_trend(metric_array)

# In[41]:

#
# metric = {}
# metric['rsq'] = 0.897
# metric['nrmse'] = 0.413
# metric['mape'] = 0.239
#
# # In[42]:
#
#
# save_metric_to_csv(metric, 'C:/Users/Federico/Downloads/KPIs.csv')
#
# # In[44]:
#
#
# coefficenti = show_coefficients(all_features, model, 'Ridge', graph=True)
#
# # In[23]:
#
#
# show_prediction_vs_actual_graph(result, 'date', 'conversions', 'prediction')
#
# # In[24]:
#
#
# pd.set_option("display.max_rows", 20)
# pd.set_option("display.max_columns", 20)
# result
#
# # In[25]:
#
#
# df_adstock_saturation, result_df_adstock = adstock_saturation_to_dataset(recommendation.value[1], 'weibull')
# save_dataset_to_csv(df_adstock_saturation, 'C:/Users/Federico/Downloads/treddom_ES_Adstock_Saturation.csv')
# df_adstock_saturation
#
# # In[27]:
#
#
# coefficenti_dict = {}
# for index, value in enumerate(coefficenti):
#     coefficenti_dict[index] = value
# coefficenti_dict['b0'] = model.intercept_
#
# # In[67]:
#
#
# # coefficenti = recommendation.value[1].copy()
# # del coefficenti['alpha']
# columns_coef = ['g_discovery_spend', 'g_display_spend', 'g_performance_max_spend', 'g_search_spend', 'g_video_spend',
#                 'fb_pr_spend', 'fb_re_spend', 'tiktok_learn_spend', 'influencer_spend',
#                 's_organic', 's_referral', 's_influencers', 's_direct', 's_others', 'keywords_impressions',
#                 # 'avg_p_cacao', 'avg_p_baobab', 'avg_p_asc2020', 'avg_p_pt2020', 'avg_p_ft_ag',
#                 'tickets_count', 'trend', 'yearly', 'monthly', 'weekly', 'holidays']
#
# # In[73]:
#
#
# df_saturation = df.copy()
#
# for index, row in df_saturation.iterrows():
#     for m in list(df_adstock_saturation['canale']):
#         df_saturation.at[index, m] = saturation(df_saturation.at[index, m], df_adstock_saturation.loc[
#             df_adstock_saturation['canale'] == m, 'saturation'].iloc[0])
#
# coef_array = coefficenti
# new_dict = {}
#
# for x in range(len(coef_array) - 1):
#     new_dict[columns_coef[x]] = df_saturation[columns_coef[x]] * coef_array[x]
#
# new_dict['intercept'] = coef_array[-1]
#
# pd.set_option("display.max_rows", 20)
# pd.set_option("display.max_columns", 20)
# df_saturation_response = pd.DataFrame(new_dict)

# In[144]:


# s_df = pd.merge(df['g_discovery_spend'], df_saturation_response['g_discovery_spend'], left_index=True, right_index=True)
# pd.set_option("display.max_rows", None)
# pd.set_option("display.max_columns", None)
# s_df


# In[145]:

#
# import plotly.express as px
#
# for m in media:
#     s_df = pd.merge(df[m], df_saturation_response[m], left_index=True, right_index=True)
#
#     # For each media remove all the 0 spend values as Log(0) can't be calculated
#     s_df[m + '_x'] = s_df[m + '_x'][s_df[m + '_x'] != 0]
#     s_df[m + '_y'] = s_df[m + '_y'][s_df[m + '_y'] != 0]
#     # saturation_df = saturation_df[(saturation_df[[media]] != 0).all(axis=1)]
#
#     # Create a Scatter Plot
#     fig = px.scatter(
#         # Our temporary DF with no 0 values
#         s_df,
#         # Media channel's name
#         x=m + '_x',
#         # Our output variable (Sales or Conversion)
#         y=m + '_y',
#         # Specify the type of trendline you want to show, use OLS if not sure
#         trendline="ols",
#         # Define the trendline as logarithmic, this will return curved functions
#         trendline_options=dict(log_x=True)
#     )
#     # Print Figure
#     fig.show()
#
# # In[158]:
#
#
# import plotly.express as px
#
# for m in media:
#     s_df = pd.merge(df[m], df_saturation_response[m], left_index=True, right_index=True)
#
#     # For each media remove all the 0 spend values as Log(0) can't be calculated
#     s_df[m + '_x'] = s_df[m + '_x'][s_df[m + '_x'] != 0]
#     s_df[m + '_y'] = s_df[m + '_y'][s_df[m + '_y'] != 0]
#     # saturation_df = saturation_df[(saturation_df[[media]] != 0).all(axis=1)]
#
#     # Create a Scatter Plot
#     fig = px.line(
#         # Our temporary DF with no 0 values
#         s_df,
#         # Media channel's name
#         x=m + '_x',
#         # Our output variable (Sales or Conversion)
#         y=m + '_y',
#         # Specify the type of trendline you want to show, use OLS if not sure
#         marker=True,
#         # Define the trendline as logarithmic, this will return curved functions
#         # trendline_options=dict(log_x=True)
#     )
#     # Print Figure
#     fig.show()
#
# # In[93]:
#
#
# df_aggregated, df_all_decomp = decomposition_to_dataset(result, coefficenti_dict, columns_coef, spends_df, media,
#                                                         'date', 'conversions', 'prediction')
# save_dataset_to_csv(df_all_decomp, 'C:/Users/Federico/Downloads/treddom_ES_Timeseries.csv')
#
# df_aggregated = pd.merge(how='left', left=df_aggregated, right=df_adstock_saturation, left_on='canale',
#                          right_on='canale')
# save_dataset_to_csv(df_aggregated, 'C:/Users/Federico/Downloads/treddom_ES_Decomposition.csv')
# df_aggregated
#
# # In[94]:
#
#
# df_all_decomp
#
# # In[95]:
#
#
# # lower_bounds = [100, spends[1]*0.6, spends[2]*0.6, spends[3]*0.6, spends[4]*0.6, spends[5]*0.6, spends[6]*0.6,
# # spends[7]*0.6, spends[8]*0.6]
# # upper_bounds = [spends[0]*1.5, spends[1]*1.5, spends[2]*1.5, spends[3]*1.5, spends[4]*1.5, spends[5]*1.5, spends[6]*1.5,
# # spends[7]*1.5, spends[8]*1.5]
#
# lower_bounds = [100, 0, 0, 50, 20, 50, 0, 0, 0]
# upper_bounds = [165, 60, 60, 165, 165, 165, 60, 60, 60]
#
# budget_allocator_df = budget_allocator(result, 'date', media, all_features, '2022-02-28',
#                                        model, spends, response_get_budget, lower_bounds, upper_bounds, 5000,
#                                        df_aggregated, algoritm='GN_ISRES')
# save_dataset_to_csv(budget_allocator_df, 'C:/Users/Federico/Downloads/treddom_ES_budget_allocator.csv')
# budget_allocator_df
#
# # In[ ]:
#
#
# # In[ ]:
#
#





# TODO
import nevergrad as ng
from cassandra.model.nevergrad.utils import instrum_trasformations_nevergrad, create_df_trasformations_nevergrad, choose_optimizer_algoritm, optimize_metric
from cassandra.model.utils import choose_model


def nevergrad_model(df, hyperparameters_dict, medias, organic, target, use_intercept=True, use_adstock=True, use_saturation=True,
                                adstock_type='weibull', model_regression='linear', X_trasformations_columns=[],
                     model_regression_log_log='linear', ridge_number=0, metric=['rsq', 'nrmse', 'mape'],
                     return_metric=True, size=0.2, positive=False, random_state=42, metric_return='mape',
              force_coeffs=False, coeffs=[], intercept=0, optimizer_algoritm='TwoPointsDE', budget=1000):

    value_trasf = instrum_trasformations_nevergrad(hyperparameters_dict)
    instrum = ng.p.Instrumentation(**value_trasf)
    optimizer = choose_optimizer_algoritm(optimizer_algoritm, budget, instrum)

    metric_array = []
    df_transformations = df.copy()
    metrics_values = {}
    all_hyper = list(hyperparameters_dict.keys())

    def build_model(**all_hyper):
        df_transf = create_df_trasformations_nevergrad(all_hyper, df, df_transformations, medias, use_adstock, use_saturation, adstock_type)
        if return_metric:
            result, model, metrics_values = choose_model(df_transf, medias + organic, target, model_regression, X_trasformations_columns,
                     model_regression_log_log, ridge_number, metric,
                     return_metric, size, positive, random_state, force_coeffs, coeffs, intercept)
        else:
            result, model = choose_model(df_transf, medias + organic, target, model_regression, X_trasformations_columns,
                     model_regression_log_log, ridge_number, metric,
                     return_metric, size, positive, random_state, force_coeffs, coeffs, intercept)

        metric_to_optimize = optimize_metric(metric_return, metrics_values)
        metric_array.append(metric_to_optimize)

        return metric_to_optimize

    recommendation = optimizer.minimize(build_model)

    def build_model_result(**all_hyper):

        df_transf = create_df_trasformations_nevergrad(all_hyper, df, df_transformations, medias, use_adstock, use_saturation, adstock_type)

        if return_metric:
            result, model, metrics_values = choose_model(df_transf, medias + organic, target, model_regression, X_trasformations_columns,
                     model_regression_log_log, ridge_number, metric,
                     return_metric, size, positive, random_state, force_coeffs, coeffs, intercept)
        else:
            result, model = choose_model(df_transf, medias + organic, target, model_regression, X_trasformations_columns,
                     model_regression_log_log, ridge_number, metric,
                     return_metric, size, positive, random_state, force_coeffs, coeffs, intercept)

        metric_to_optimize = optimize_metric(metric_return, metrics_values)
        metric_array.append(metric_to_optimize)

        return metric_to_optimize, result, model

    metric, result, model = build_model_result(**recommendation.value[1])
    return metric, result, model, metric_array







import pandas as pd
import numpy as np
from cassandra.data.trasformations.trasformations import saturation
from cassandra.budgetAllocator.utils import get_opt_algoritm


def budget_allocator_c(df, name_date_column, medias, all_features, date_get_budget, model, spends, response_get_budget,
                       lower_bounds, upper_bounds, maxeval, df_aggregated, algoritm='LD_MMA', number_tail=15):

    def getVals(df, name_date_column, all_features, date_get_budget='', number_tail=number_tail):
        if date_get_budget:
            get_vals_df = df.copy()
            get_vals_df.drop(get_vals_df[get_vals_df[name_date_column] > date_get_budget].index)
            full_row = pd.DataFrame(get_vals_df.tail(number_tail).mean()).T
        else:
            full_row = df.tail(number_tail).mean()

        row = full_row[all_features].copy()

        return row

    def myFunc(x, grad=[]):

        data = {}

        for m in medias:
            data[m] = [saturation(x[medias.index(m)], df_aggregated.loc[
                df_aggregated['canale'] == m, 'saturation'].iloc[0])]

        dic = pd.DataFrame.from_dict(data)

        new_df = getVals(df, name_date_column, all_features, date_get_budget, number_tail).copy()

        for column in dic:
            new_df[column] = dic[column].iloc[0]

        return model.predict(new_df)[0]

    opt = get_opt_algoritm(algoritm, medias, lower_bounds, upper_bounds, spends, maxeval, myFunc)

    budget_spends = opt.optimize(spends)

    budget_allocator_df = pd.DataFrame()
    budget_allocator_df['canale'] = medias
    for index, row in budget_allocator_df.iterrows():
        budget_allocator_df.at[index, 'actual_spend'] = spends[index]
        budget_allocator_df.at[index, 'optimal_spend'] = budget_spends[index]
        budget_allocator_df.at[index, 'actual_response'] = df_aggregated.loc[index, 'xDecompAgg']
        budget_allocator_df.at[index, 'optimal_response'] = budget_spends[index] * df_aggregated.loc[index, 'coef']
        budget_allocator_df.at[index, 'actual_total_spend'] = np.sum(spends)
        budget_allocator_df.at[index, 'optimal_total_spend'] = np.sum(budget_spends)
        budget_allocator_df.at[index, 'actual_total_response'] = response_get_budget
        budget_allocator_df.at[index, 'optimal_total_response'] = opt.last_optimum_value()

    return budget_allocator_df

    # def getVals(df, name_date_column, all_features, date_get_budget):

    # full_row = df.loc[df[name_date_column] == date_get_budget]
    # print(full_row)
    # row = full_row[all_features].copy()
    # print(row)

    # return row



from dataProcessing.cleanFormatMerge import guess_categorical_variables, guess_numerical_variables
from sklearn.compose import ColumnTransformer
from sklearn.model_selection import train_test_split, GridSearchCV
from sklearn.neural_network import MLPRegressor
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import OneHotEncoder, QuantileTransformer, Normalizer
from cassandra.model.modelEvaluation.plot import show_nrmse, show_mape, show_rsquared


def deepLearning(df, X_columns, target, name_model, metric=['rsq', 'nrmse', 'mape'], return_metric=False, cv=5,
                 verbose=2, size=0.2, random_state=42, force_coeffs=False, coeffs=[], intercept=0):
    metrics_values = {}
    X = df[X_columns]
    y = df[target]

    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=size, random_state=random_state)
    all_features = list(X_train.columns)
    categorical = guess_categorical_variables(X_train)
    numerical = guess_numerical_variables(X_train.drop(categorical, axis=1))

    transformers = [
        ('one hot', OneHotEncoder(handle_unknown='ignore'), categorical),
        ('scaler', QuantileTransformer(), numerical),
        ('normalizer', Normalizer(), all_features)
    ]
    ct = ColumnTransformer(transformers)

    if len(df.index) < 1000:
        solver_value = 'lbfgs'
    else:
        solver_value = 'adam'

    steps = [
        ('column_transformer', ct),
        ('model', MLPRegressor(solver=solver_value))
        # solver 'lbfgs' is used for dataset with less than 1000 rows, if more than 1000 use solver 'adam'
    ]
    pipeline = Pipeline(steps)
    param_space = {
        'column_transformer__scaler__n_quantiles': [80, 100, 120],
        'column_transformer__normalizer': [Normalizer(), 'passthrough'],
        'model__hidden_layer_sizes': [(35, 35), (50, 50), (75, 75)],
        'model__alpha': [0.005, 0.001]
    }

    # input the param space into "param_grid", define what pipeline it needs to run, in our case is named "pipeline", and the you can decide how many cross validation can do "cv=" and the verbosity.
    model = GridSearchCV(pipeline, param_grid=param_space, cv=cv, verbose=verbose)
    model.fit(X_train, y_train)

    if force_coeffs and coeffs:
        model.coef_ = coeffs
        model.intercept_ = intercept

    # model.best_estimator_

    # Ask the model to predict on X_test without having Y_test
    # This will give you exact predicted values

    # We can use our NRMSE and MAPE functions as well

    # Create new DF not to edit the original one
    result = df

    # Create a new column with predicted values
    result['prediction'] = model.predict(result)

    # Score returns the accuracy of the above prediction or R^2
    if 'rsq' in metric:
        rsq = show_rsquared(result[target], result['prediction'])
        if return_metric:
            metrics_values[name_model + '_rsq'] = rsq
        print(name_model, 'RSQ: ', rsq)

    # Get the NRMSE & MAPE values
    if 'nrmse' in metric:
        nrmse_val = show_nrmse(result[target], result['prediction'])
        if return_metric:
            metrics_values[name_model + '_nrmse'] = nrmse_val
        print(name_model, 'NRMSE: ', nrmse_val)

    if 'mape' in metric:
        mape_val = show_mape(result[target], result['prediction'])
        if return_metric:
            metrics_values[name_model + '_mape'] = mape_val
        print(name_model, 'MAPE: ', mape_val)

    if metrics_values:
        return result, model, metrics_values
    else:
        return result, model



# TODO
import statsmodels.formula.api as smf


# Un example of y_and_X variable
# "conversions ~ date + fb_pr_spend + fb_re_spend + g_discovery_spend + g_display_spend + g_performance_max_spend + g_search_spend + g_video_spend"
def mlm(df, y_and_X, groups_column_name, re_formula=''):
    model = smf.mixedlm(y_and_X, df, groups=df[groups_column_name], re_formula=re_formula)

    result = model.fit()

    print(result.summary())

    return result, model



# TODO
from cassandra.data.trasformations.trasformations import adstock_weibull, saturation, adstock
import nevergrad as ng
import numpy as np


def create_df_trasformations_nevergrad(trasf, df, df_transformations, medias, use_adstock = True, use_saturation = True, adstock_type = 'weibull'):

    #key_trasf = list(hyperparameters_dict.keys())
    #trasf = dict(zip(key_trasf, value_trasf))

    if use_saturation:
        if use_adstock:
            if adstock_type == 'weibull':
                for m in medias:
                    df_transformations[m] = saturation(adstock_weibull(df[m], trasf[m + '_scale'], trasf[m + '_shape']).replace([np.inf, -np.inf], 0), trasf[m + '_beta'])

            else:
                for m in medias:
                    df_transformations[m] = saturation(adstock(df[m], trasf[m + '_theta']), trasf[m + '_beta'])
        else:
            for m in medias:
                df_transformations[m] = saturation(df[m], trasf[m + '_beta'])

    else:
        if use_adstock:
            if adstock_type == 'weibull':
                for m in medias:
                    df_transformations[m] = saturation(
                        adstock_weibull(df[m], trasf[m + '_scale'], trasf[m + '_shape']).replace([np.inf, -np.inf],
                                                                                                 0),
                        trasf[m + '_beta'])

            else:
                for m in medias:
                    df_transformations[m] = saturation(adstock(df[m], trasf[m + '_theta']), trasf[m + '_beta'])

    return df_transformations


def instrum_variables_nevergrad(medias, organic, force_coeffs=False, use_intercept=True, use_adstock=True, use_saturation=True,
                                adstock_type='weibull'):
    result_dict = {}

    if force_coeffs:
        for x in medias:
            result_dict[x] = {'lower': 0, 'upper': None}
        for x in organic:
            result_dict[x] = {'lower': None, 'upper': None}
        if use_intercept:
            result_dict['intercept'] = {'lower': None, 'upper': None}

    if use_adstock:
        if adstock_type == 'weibull':
            for x in medias:
                result_dict[x + '_shape'] = {'lower': 0, 'upper': 2}
                result_dict[x + '_scale'] = {'lower': 0, 'upper': 0.1}

                if use_saturation:
                    result_dict[x + '_beta'] = {'lower': 0, 'upper': 1}
        else:
            for x in medias:
                result_dict[x + '_theta'] = {'lower': 0, 'upper': 0.3}

                if use_saturation:
                    result_dict[x + '_beta'] = {'lower': 0, 'upper': 1}

    return result_dict


def instrum_trasformations_nevergrad(hyperparameters_dict):
    result = {}
    for key, value in hyperparameters_dict.items():
        result[key] = ng.p.Scalar(lower=hyperparameters_dict[key]['lower'], upper=hyperparameters_dict[key]['upper'])

    return result

def choose_optimizer_algoritm(optimizer_algoritm, budget, instrum):
    if optimizer_algoritm == 'TwoPointsDE':
        optimizer = ng.optimizers.TwoPointsDE(parametrization=instrum, budget=budget)
    if optimizer_algoritm == 'NGOpt':
        optimizer = ng.optimizers.NGOpt(parametrization=instrum, budget=budget)

    return optimizer

def optimize_metric(return_metric, metrics_values):
    result = ''
    for key, value in metrics_values.items():
        if return_metric in key:
            result = value

    return result






import numpy as np

#' Automatic function for the extraction of categorical variables
def categorical_columns(df):
    cat_vars = []
    for x in df.columns:
        values = list(df[x].value_counts().index)
        if values in [[0, 1], [1], [True, False], [True]]:
            cat_vars.append(x)

    return cat_vars

#' Automatic function for the extraction of numerical variables
def numerical_columns(df):
    return list(df.select_dtypes(include=[np.number]).columns.values)


import nlopt
import numpy as np


def getBudget(df, name_date_column, name_target_column, medias, date_get_budget='', number_tail=15):
    if date_get_budget:
        new_df = df.copy()
        new_df.drop(new_df[new_df[name_date_column] > date_get_budget].index, inplace=True)
        full_row = new_df.tail(number_tail).mean()
    else:
        full_row = df.tail(number_tail).mean()

    spends = []

    for m in medias:
        spends.append(full_row[m])

    response = full_row[name_target_column]
    return spends, response


def getVals(df, name_date_column, all_features, date_get_budget='', number_tail=15):
    if date_get_budget:
        new_df = df.copy()
        new_df.drop(new_df[new_df[name_date_column] > date_get_budget].index, inplace=True)
        full_row = new_df.tail(number_tail).mean()
    else:
        full_row = df.tail(number_tail).mean()

    row = full_row[all_features].copy()

    return row


def get_opt_algoritm(algoritm, medias, lower_bounds, upper_bounds, spends, maxeval, myFunc):
    if algoritm in ['GN_DIRECT', 'LD_SLSQP', 'GN_CRS2_LM', 'GD_STOGO', 'LN_BOBYQA', 'LN_NEWUOA', 'LD_CCSAQ']:
        if algoritm == 'GN_DIRECT':
            opt = nlopt.opt(nlopt.GN_DIRECT, len(medias))

        elif algoritm == 'LD_SLSQP':
            opt = nlopt.opt(nlopt.LD_SLSQP, len(medias))

        elif algoritm == 'GN_CRS2_LM':
            opt = nlopt.opt(nlopt.GN_CRS2_LM, len(medias))

        elif algoritm == 'GD_STOGO':
            opt = nlopt.opt(nlopt.GD_STOGO, len(medias))

        elif algoritm == 'LN_BOBYQA':
            opt = nlopt.opt(nlopt.LN_BOBYQA, len(medias))

        elif algoritm == 'LN_NEWUOA':
            opt = nlopt.opt(nlopt.LN_NEWUOA, len(medias))

        else:
            opt = nlopt.opt(nlopt.LD_CCSAQ, len(medias))

        lower_boundaries = np.multiply(lower_bounds, spends)
        upper_boundaries = np.multiply(upper_bounds, spends)

        opt.set_lower_bounds(lower_boundaries)
        opt.set_upper_bounds(upper_boundaries)

        opt.set_max_objective(myFunc)

        opt.set_xtol_rel(1e-14)
        opt.set_maxeval(maxeval)

    else:
        if algoritm == 'GN_ISRES':
            opt = nlopt.opt(nlopt.GN_ISRES, len(medias))

        elif algoritm == 'GN_AGS':
            opt = nlopt.opt(nlopt.GN_AGS, len(medias))

        elif algoritm == 'LN_COBYLA':
            opt = nlopt.opt(nlopt.LN_COBYLA, len(medias))

        elif algoritm == 'GN_ESCH':
            opt = nlopt.opt(nlopt.GN_ESCH, len(medias))

        elif algoritm == 'AUGLAG':
            opt = nlopt.opt(nlopt.AUGLAG, len(medias))

        else:
            opt = nlopt.opt(nlopt.LD_MMA, len(medias))

        lower_boundaries = np.multiply(lower_bounds, spends)
        upper_boundaries = np.multiply(upper_bounds, spends)

        opt.set_lower_bounds(lower_boundaries)
        opt.set_upper_bounds(upper_boundaries)

        opt.set_max_objective(myFunc)
        opt.add_inequality_constraint(lambda z, grad: sum(z) - np.sum(spends), 1e-8)

        # rate of improvement, below which we are done
        opt.set_xtol_rel(1e-14)
        opt.set_maxeval(maxeval)

    return opt



"""The setup script."""
import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent
VERSION = "1.0.0"
PACKAGE_NAME = "cassandra_mmm"
AUTHOR = "Cassandra"
URL = "https://github.com/Tsurty/cassandra"
LICENSE = "MIT"
DESCRIPTION = "Cassandra library"
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf8')
LONG_DESC_TYPE = "text/markdown"
#INSTALL_REQUIRES = (HERE / "requirements.txt").read_text(encoding='utf8')

setup(name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    license=LICENSE,
    #author_email=AUTHOR_EMAIL,
    url=URL,
    #install_requires=INSTALL_REQUIRES,
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    package_data = {
    '': ['*.txt', '*.csv'],
    },
    include_package_data=True
)


import pandas as pd
from cassandra.model.modelEvaluation.evaluation import response_regression_to_dataset, decomposition_to_dataset, adstock_saturation_to_dataset, spend_to_dataset


def save_metric_to_csv(metric_dictionary, path_file_to_save):
    my_dict = [metric_dictionary]
    df = pd.DataFrame.from_dict(my_dict)
    df.to_csv(path_file_to_save, index = False, header=True)

def save_timeseries_to_csv(df, name_date_column, name_target_colum, name_prediction_column, coef_dict, features, path_file_to_save):
    response_df = response_regression_to_dataset(df, name_date_column, name_target_colum, name_prediction_column, coef_dict,
                                   features)
    response_df.to_csv(path_file_to_save, index = False, header=True)

def save_decomposition_to_csv(df, coef_dict, features, spend_df, medias, path_file_to_save):
    response_df = decomposition_to_dataset(df, coef_dict, features, spend_df, medias)
    response_df.to_csv(path_file_to_save, index = False, header=True)

def save_adstock_saturation_to_csv(dict_theta_and_beta, path_file_to_save):
    response_df = adstock_saturation_to_dataset(dict_theta_and_beta)
    response_df.to_csv(path_file_to_save, index = False, header=True)

def save_spend_to_csv(df, medias, path_file_to_save):
    response_df = spend_to_dataset(df, medias)
    response_df.to_csv(path_file_to_save, index = False, header=True)

def save_dataset_to_csv(df, path_file_to_save):
    df.to_csv(path_file_to_save, index = False, header=True)